@@ -181,6 +181,7 @@ coef.summary.dynrCook <- function(object, ...){
 ##' element of the (transformed, inverse) Hessian was negative and an absolute value was taken to make it positive.
 ##' 
 ##' @method summary dynrCook
+##' @return Object of class summary.dynrCook.  Primarily used for showing the results of a fitted model.
 summary.dynrCook <- summaryResults
 
 
@@ -224,8 +225,43 @@ setMethod("show", "dynrCook", function(object) {
 ##' @seealso Other S3 methods \code{\link{logLik.dynrCook}}
 ##' 
 ##' @examples
-##' # Let cookedModel be the output from dynr.cook
-##' #coef(cookedModel)
+##' # Create a minimal cooked model called 'cook'
+##' require(dynr)
+##' 
+##' meas <- prep.measurement(
+##' 	values.load=matrix(c(1, 0), 1, 2),
+##' 	params.load=matrix(c('fixed', 'fixed'), 1, 2),
+##' 	state.names=c("Position","Velocity"),
+##' 	obs.names=c("y1"))
+##' 
+##' ecov <- prep.noise(
+##' 	values.latent=diag(c(0, 1), 2),
+##' 	params.latent=diag(c('fixed', 'dnoise'), 2),
+##' 	values.observed=diag(1.5, 1),
+##' 	params.observed=diag('mnoise', 1))
+##' 
+##' initial <- prep.initial(
+##' 	values.inistate=c(0, 1),
+##' 	params.inistate=c('inipos', 'fixed'),
+##' 	values.inicov=diag(1, 2),
+##' 	params.inicov=diag('fixed', 2))
+##' 
+##' dynamics <- prep.matrixDynamics(
+##' 	values.dyn=matrix(c(0, -0.1, 1, -0.2), 2, 2),
+##' 	params.dyn=matrix(c('fixed', 'spring', 'fixed', 'friction'), 2, 2),
+##' 	isContinuousTime=TRUE)
+##' 
+##' data(Oscillator)
+##' data <- dynr.data(Oscillator, id="id", time="times", observed="y1")
+##' 
+##' model <- dynr.model(dynamics=dynamics, measurement=meas,
+##' 	noise=ecov, initial=initial, data=data)
+##' 
+##' cook <- dynr.cook(model,
+##' 	verbose=FALSE, optimization_flag=FALSE, hessian_flag=FALSE)
+##' 
+##' # Now grab the coef!
+##' coef(cook)
 coef.dynrCook <- function(object, ...){
 	object@transformed.parameters
 }
@@ -252,8 +288,43 @@ coef.dynrCook <- function(object, ...){
 ##' @seealso Other S3 methods \code{\link{coef.dynrCook}}
 ##' 
 ##' @examples
-##' # Let cookedModel be the output from dynr.cook
-##' #logLik(cookedModel)
+##' # Minimal model
+##' require(dynr)
+##' 
+##' meas <- prep.measurement(
+##' 	values.load=matrix(c(1, 0), 1, 2),
+##' 	params.load=matrix(c('fixed', 'fixed'), 1, 2),
+##' 	state.names=c("Position","Velocity"),
+##' 	obs.names=c("y1"))
+##' 
+##' ecov <- prep.noise(
+##' 	values.latent=diag(c(0, 1), 2),
+##' 	params.latent=diag(c('fixed', 'dnoise'), 2),
+##' 	values.observed=diag(1.5, 1),
+##' 	params.observed=diag('mnoise', 1))
+##' 
+##' initial <- prep.initial(
+##' 	values.inistate=c(0, 1),
+##' 	params.inistate=c('inipos', 'fixed'),
+##' 	values.inicov=diag(1, 2),
+##' 	params.inicov=diag('fixed', 2))
+##' 
+##' dynamics <- prep.matrixDynamics(
+##' 	values.dyn=matrix(c(0, -0.1, 1, -0.2), 2, 2),
+##' 	params.dyn=matrix(c('fixed', 'spring', 'fixed', 'friction'), 2, 2),
+##' 	isContinuousTime=TRUE)
+##' 
+##' data(Oscillator)
+##' data <- dynr.data(Oscillator, id="id", time="times", observed="y1")
+##' 
+##' model <- dynr.model(dynamics=dynamics, measurement=meas,
+##' 	noise=ecov, initial=initial, data=data)
+##' 
+##' cook <- dynr.cook(model,
+##' 	verbose=FALSE, optimization_flag=FALSE, hessian_flag=FALSE)
+##' 
+##' # Now get the log likelihood!
+##' logLik(cook)
 logLik.dynrCook <- function(object, ...){
 	ans <- -object@neg.log.likelihood
 	attr(ans, "df") <- length(object@fitted.parameters)
@@ -282,8 +353,43 @@ deviance.dynrCook <- function(object, ...){
 ##' A single number. The total number of observations across all IDs.
 ##' 
 ##' @examples
-##' # Let cookedModel be the output from dynr.cook
-##' #nobs(cookedModel)
+##' # Minimal model
+##' require(dynr)
+##' 
+##' meas <- prep.measurement(
+##' 	values.load=matrix(c(1, 0), 1, 2),
+##' 	params.load=matrix(c('fixed', 'fixed'), 1, 2),
+##' 	state.names=c("Position","Velocity"),
+##' 	obs.names=c("y1"))
+##' 
+##' ecov <- prep.noise(
+##' 	values.latent=diag(c(0, 1), 2),
+##' 	params.latent=diag(c('fixed', 'dnoise'), 2),
+##' 	values.observed=diag(1.5, 1),
+##' 	params.observed=diag('mnoise', 1))
+##' 
+##' initial <- prep.initial(
+##' 	values.inistate=c(0, 1),
+##' 	params.inistate=c('inipos', 'fixed'),
+##' 	values.inicov=diag(1, 2),
+##' 	params.inicov=diag('fixed', 2))
+##' 
+##' dynamics <- prep.matrixDynamics(
+##' 	values.dyn=matrix(c(0, -0.1, 1, -0.2), 2, 2),
+##' 	params.dyn=matrix(c('fixed', 'spring', 'fixed', 'friction'), 2, 2),
+##' 	isContinuousTime=TRUE)
+##' 
+##' data(Oscillator)
+##' data <- dynr.data(Oscillator, id="id", time="times", observed="y1")
+##' 
+##' model <- dynr.model(dynamics=dynamics, measurement=meas,
+##' 	noise=ecov, initial=initial, data=data)
+##' 
+##' cook <- dynr.cook(model,
+##' 	verbose=FALSE, optimization_flag=FALSE, hessian_flag=FALSE)
+##' 
+##' # Now get the total number of observations
+##' nobs(cook)
 nobs.dynrCook <- function(object, ...){
 	dim(object@eta_smooth_final)[2]
 }
@@ -298,6 +404,8 @@ nobs.dynrCook <- function(object, ...){
 ##' 
 ##' @details
 ##' This is the inverse Hessian of the transformed parameters.
+##'
+##' @return matrix.  Asymptotic variance-covariance matrix of the transformed parameters.
 vcov.dynrCook <- function(object, ...){
 	nm <- names(coef(object))
 	rt <- object@transformed.inv.hessian
@@ -367,8 +475,44 @@ setMethod("$", "dynrCook",
 ##' Wu, H. & Neale, M. C. (2012). Adjusted confidence intervals for a bounded parameter. Behavior genetics, 42(6), 886-898.
 ##' 
 ##' @examples
-##' # Let cookedModel be the output from dynr.cook
-##' #confint(cookedModel)
+##' # Minimal model
+##' require(dynr)
+##' 
+##' meas <- prep.measurement(
+##' 	values.load=matrix(c(1, 0), 1, 2),
+##' 	params.load=matrix(c('fixed', 'fixed'), 1, 2),
+##' 	state.names=c("Position","Velocity"),
+##' 	obs.names=c("y1"))
+##' 
+##' ecov <- prep.noise(
+##' 	values.latent=diag(c(0, 1), 2),
+##' 	params.latent=diag(c('fixed', 'dnoise'), 2),
+##' 	values.observed=diag(1.5, 1),
+##' 	params.observed=diag('mnoise', 1))
+##' 
+##' initial <- prep.initial(
+##' 	values.inistate=c(0, 1),
+##' 	params.inistate=c('inipos', 'fixed'),
+##' 	values.inicov=diag(1, 2),
+##' 	params.inicov=diag('fixed', 2))
+##' 
+##' dynamics <- prep.matrixDynamics(
+##' 	values.dyn=matrix(c(0, -0.1, 1, -0.2), 2, 2),
+##' 	params.dyn=matrix(c('fixed', 'spring', 'fixed', 'friction'), 2, 2),
+##' 	isContinuousTime=TRUE)
+##' 
+##' data(Oscillator)
+##' data <- dynr.data(Oscillator, id="id", time="times", observed="y1")
+##' 
+##' model <- dynr.model(dynamics=dynamics, measurement=meas,
+##' 	noise=ecov, initial=initial, data=data)
+##' 
+##' cook <- dynr.cook(model,
+##' 	verbose=FALSE, optimization_flag=FALSE, hessian_flag=FALSE)
+##' 
+##' # Now get the confidence intervals
+##' # But note that they are nonsense because we set hessian_flag=FALSE !!!!
+##' confint(cook)
 confint.dynrCook <- function(object, parm, level = 0.95, type = c("delta.method", "endpoint.transformation"), transformation =  NULL, ...){
 	type <- match.arg(type)
 	tlev <- (1-level)/2
@@ -451,6 +595,8 @@ confint.dynrCook <- function(object, parm, level = 0.95, type = c("delta.method"
 ##' }
 ##' The last row of this table corresponding to an exit code of -6, is not from NLOPT, but rather is specific to the \code{dynr} package.
 ##' 
+##' @return Object of class dynrCook.
+##' 
 ##' @seealso 
 ##' \code{\link{autoplot}}, \code{\link{coef}}, \code{\link{confint}},
 ##' \code{\link{deviance}}, \code{\link{initialize}}, \code{\link{logLik}},
@@ -458,7 +604,41 @@ confint.dynrCook <- function(object, parm, level = 0.95, type = c("delta.method"
 ##' \code{\link{show}}, \code{\link{summary}}, \code{\link{vcov}}.          
 ##' 
 ##' @examples
-##' #fitted.model <- dynr.cook(model)
+##' # Minimal model
+##' require(dynr)
+##' 
+##' meas <- prep.measurement(
+##' 	values.load=matrix(c(1, 0), 1, 2),
+##' 	params.load=matrix(c('fixed', 'fixed'), 1, 2),
+##' 	state.names=c("Position","Velocity"),
+##' 	obs.names=c("y1"))
+##' 
+##' ecov <- prep.noise(
+##' 	values.latent=diag(c(0, 1), 2),
+##' 	params.latent=diag(c('fixed', 'dnoise'), 2),
+##' 	values.observed=diag(1.5, 1),
+##' 	params.observed=diag('mnoise', 1))
+##' 
+##' initial <- prep.initial(
+##' 	values.inistate=c(0, 1),
+##' 	params.inistate=c('inipos', 'fixed'),
+##' 	values.inicov=diag(1, 2),
+##' 	params.inicov=diag('fixed', 2))
+##' 
+##' dynamics <- prep.matrixDynamics(
+##' 	values.dyn=matrix(c(0, -0.1, 1, -0.2), 2, 2),
+##' 	params.dyn=matrix(c('fixed', 'spring', 'fixed', 'friction'), 2, 2),
+##' 	isContinuousTime=TRUE)
+##' 
+##' data(Oscillator)
+##' data <- dynr.data(Oscillator, id="id", time="times", observed="y1")
+##' 
+##' model <- dynr.model(dynamics=dynamics, measurement=meas,
+##' 	noise=ecov, initial=initial, data=data)
+##' 
+##' # Now cook the model!
+##' cook <- dynr.cook(model,
+##' 	verbose=FALSE, optimization_flag=FALSE, hessian_flag=FALSE)
 dynr.cook <- function(dynrModel, conf.level=.95, infile, optimization_flag=TRUE, hessian_flag = TRUE, verbose=TRUE, weight_flag=FALSE, debug_flag=FALSE, perturb_flag=FALSE) {
 	frontendStart <- Sys.time()
 	transformation=dynrModel@transform@tfun
@@ -471,18 +651,18 @@ dynr.cook <- function(dynrModel, conf.level=.95, infile, optimization_flag=TRUE,
 		# Always use 'verbose' function argument but only say so when they disagree and verbose=TRUE.
 	}
 	
-	if (.hasSlot(dynrModel$dynamics, 'theta.formula') && length(dynrModel$dynamics@theta.formula) > 0){
-		#get the initial values of b and startvars
-		fitted_model <- EstimateRandomAsLV(dynrModel, optimization_flag, hessian_flag, verbose, weight_flag, debug_flag)
-		coefEst <- coef(fitted_model)
-		estimated.names <- intersect(names(dynrModel@xstart), names(coefEst))
-		dynrModel@xstart[estimated.names] <- coefEst[estimated.names]
-		#print('Starting values:')
-		#print(dynrModel@xstart)
-		#if('b_est' %in% names(fitted_model))
-		#	b <- fitted_model@b_est
-		return(fitted_model)
-	}	#internalModelPrep convert dynrModel to a model list
+	# if (.hasSlot(dynrModel$dynamics, 'theta.formula') && length(dynrModel$dynamics@theta.formula) > 0){
+		# #get the initial values of b and startvars
+		# fitted_model <- EstimateRandomAsLV(dynrModel$extended_model, optimization_flag, hessian_flag, verbose, weight_flag, debug_flag)
+		# coefEst <- coef(fitted_model)
+		# estimated.names <- intersect(names(dynrModel@xstart), names(coefEst))
+		# dynrModel@xstart[estimated.names] <- coefEst[estimated.names]
+		# #print('Starting values:')
+		# #print(dynrModel@xstart)
+		# #if('b_est' %in% names(fitted_model))
+		# #	b <- fitted_model@b_est
+		# return(fitted_model)
+	# }	#internalModelPrep convert dynrModel to a model list
 	
 	model <- internalModelPrep(
 		num_regime=dynrModel@num_regime,
@@ -896,7 +1076,9 @@ sechol <- function(A, tol = .Machine$double.eps, silent= TRUE ) {
 
 # the function to extend the user-specified model to include random varibles
 # currently extend all b at a time
-EstimateRandomAsLV<- function(dynrModel, optimization_flag=TRUE, hessian_flag = TRUE, verbose=TRUE, weight_flag=FALSE, debug_flag=FALSE){  
+# return the extended model but does not cook
+#EstimateRandomAsLVRModel<- function(dynrModel, optimization_flag=TRUE, hessian_flag = TRUE, verbose=TRUE, weight_flag=FALSE, debug_flag=FALSE){
+EstimateRandomAsLVModel<- function(dynrModel){  
     # Restructure mixed effects structured via theta.formula into an expanded model with 
     # random effects as additional state variables and cook it.
     #browser()
@@ -927,12 +1109,13 @@ EstimateRandomAsLV<- function(dynrModel, optimization_flag=TRUE, hessian_flag =
     }
     #browser()
     # If there is random effect to be estimated, set up a new model
+    params.latent = diag(c(diag(dynrModel@noise@params.latent[[1]]), rep(0, length(user.random.names))))
     mdcov2 <- prep.noise(
-        values.latent=diag(0, length(state.names2)),
+        values.latent=diag(c(diag(dynrModel@noise@values.latent[[1]]), rep(0, length(user.random.names)))),
+        params.latent=matrix(mapply(function(x) {if(x > 0){return(dynrModel@param.names[x])} else{return("fixed")}}, params.latent), nrow=nrow(params.latent)),
         #params.latent=diag(rep("fixed",length(state.names2)), length(state.names2)),
         #params.latent=diag(state.names2, length(state.names2)),
         #params.latent=diag(c(diag(dynrModel@noise@params.latent[[1]]), rep('fixed',length(user.random.names)))),
-	params.latent = params.latent(),
         values.observed=dynrModel@noise@values.observed[[1]],
         params.observed=matrix(mapply(function(x) {if(x > 0){return(dynrModel@param.names[x])} else{return("fixed")}}, dynrModel@noise@params.observed[[1]]), nrow=nrow(dynrModel@noise@params.observed[[1]]))
         #params.observed=dynrModel@noise@params.observed[[1]]
@@ -1009,10 +1192,11 @@ EstimateRandomAsLV<- function(dynrModel, optimization_flag=TRUE, hessian_flag =
                          noise=mdcov2, initial=initial2, data=dynrModel@data,
                          outfile=dynrModel@outfile)
 
-    fitted_model2 <- dynr.cook(model2, optimization_flag=optimization_flag, hessian_flag = hessian_flag, verbose=verbose, weight_flag=weight_flag, debug_flag=debug_flag)
+    #fitted_model2 <- dynr.cook(model2, optimization_flag=optimization_flag, hessian_flag = hessian_flag, verbose=verbose, weight_flag=weight_flag, debug_flag=debug_flag)
     
     
     
-    return(fitted_model2)
+    #return(fitted_model2)
+	return(model2)
     
 }
